==> fl_camera-2.2.1.tar.gz: example/lib/camera_page.dart <==
import 'package:example/main.dart';
import 'package:fl_camera/fl_camera.dart';
import 'package:flutter/material.dart';
import 'package:flutter_waya/flutter_waya.dart';

class CameraPage extends StatefulWidget {
  const CameraPage({Key? key}) : super(key: key);

  @override
  _CameraPageState createState() => _CameraPageState();
}

class _CameraPageState extends State<CameraPage> {
  bool flashState = false;
  double ratio = 1;
  double? maxRatio;
  StateSetter? flashZoomState;

  ///  The first rendering is null ，Using the rear camera
  CameraInfo? currentCamera;
  bool isBcakCamera = true;

  ValueNotifier<FlCameraController?> controller =
      ValueNotifier<FlCameraController?>(null);

  ValueNotifier<bool> hasPreview = ValueNotifier<bool>(false);

  void listener() {
    if (hasPreview.value != controller.value!.hasPreview) {
      hasPreview.value = controller.value!.hasPreview;
    }
  }

  @override
  Widget build(BuildContext context) {
    return ExtendedScaffold(
        body: Stack(children: <Widget>[
      FlCamera(
          onCreateView: (FlCameraController _controller) {
            controller.value = _controller;
            controller.value!.addListener(listener);
          },
          uninitialized: Container(
              color: Colors.black,
              alignment: Alignment.center,
              child: const Text('Camera uninitialized',
                  style: TextStyle(color: Colors.white))),
          fit: BoxFit.fitWidth,
          onFlashChanged: (FlashState state) {
            showToast('flash state:$state');
          },
          onZoomChanged: (CameraZoomState zommState) {
            showToast('zoom ratio:${zommState.zoomRatio}');
            if (maxRatio == null && flashZoomState != null) {
              maxRatio = zommState.maxZoomRatio;
              flashZoomState!(() {});
            }
          },
          overlay: Align(
              alignment: Alignment.bottomCenter,
              child: StatefulBuilder(builder: (_, StateSetter state) {
                flashZoomState = state;
                return Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Slider(
                          value: ratio,
                          min: 1,
                          max: maxRatio ?? 20,
                          onChanged: (double value) async {
                            ratio = value;
                            flashZoomState!(() {});
                            controller.value?.setZoomRatio(value);
                          }),
                      IconBox(
                          size: 30,
                          color: flashState
                              ? Colors.white
                              : Colors.white.withOpacity(0.6),
                          padding: const EdgeInsets.fromLTRB(10, 10, 10, 40),
                          icon: flashState ? Icons.flash_on : Icons.flash_off,
                          onTap: () async {
                            final bool? state = await controller.value
                                ?.setFlashMode(flashState
                                    ? FlashState.off
                                    : FlashState.on);
                            if (state == true) {
                              flashState = !flashState;
                              flashZoomState!(() {});
                            }
                          })
                    ]);
              }))),
      Positioned(
          right: 12,
          left: 12,
          top: getStatusBarHeight + 12,
          child: ValueListenableBuilder<FlCameraController?>(
              valueListenable: controller,
              builder: (_, FlCameraController? flCameraController, __) {
                return flCameraController == null
                    ? const SizedBox()
                    : Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                            const BackButton(
                                color: Colors.white, onPressed: pop),
                            Row(children: [
                              ElevatedIcon(
                                  icon: Icons.flip_camera_ios,
                                  onPressed: switchCamera),
                              const SizedBox(width: 12),
                              ElevatedText(
                                  text: 'reset',
                                  onPressed: () =>
                                      controller.value?.resetCamera()),
                              const SizedBox(width: 12),
                              previewButton(flCameraController),
                            ])
                          ]);
              })),
    ]));
  }

  Widget previewButton(FlCameraController flCameraController) {
    return ValueListenableBuilder(
        valueListenable: hasPreview,
        builder: (_, bool hasPreview, __) {
          return ElevatedText(
            text: !hasPreview ? 'start' : 'stop',
            onPressed: () async {
              if (!hasPreview) {
                if (flCameraController.previousCamera != null) {
                  await flCameraController
                      .startPreview(flCameraController.previousCamera!);
                }
              } else {
                await flCameraController.stopPreview();
              }
            },
          );
        });
  }

  Future<void> switchCamera() async {
    if (controller.value == null) return;
    for (final CameraInfo cameraInfo in controller.value!.cameras!) {
      if (cameraInfo.lensFacing ==
          (isBcakCamera ? CameraLensFacing.front : CameraLensFacing.back)) {
        currentCamera = cameraInfo;
        break;
      }
    }
    await controller.value!.switchCamera(currentCamera!);
    isBcakCamera = !isBcakCamera;
  }

  @override
  void dispose() {
    super.dispose();
    controller.dispose();
    hasPreview.dispose();
  }
}

class ScannerBoxPage extends StatelessWidget {
  const ScannerBoxPage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const ExtendedScaffold(
        body: FlCamera(
            overlay: ScannerBox(
                borderColor: Colors.blue,
                scannerColor: Colors.blue,
                scannerSize: Size(300, 300),
                scannerStrokeWidth: 2)));
  }
}

class ScannerLinePage extends StatelessWidget {
  const ScannerLinePage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const ExtendedScaffold(
        body: FlCamera(overlay: ScannerLine(color: Colors.blue)));
  }
}

==> fl_camera-3.0.0.tar.gz: example/lib/camera_page.dart <==
import 'package:example/main.dart';
import 'package:fl_camera/fl_camera.dart';
import 'package:flutter/material.dart';
import 'package:flutter_waya/flutter_waya.dart';

class CameraPage extends StatefulWidget {
  const CameraPage({Key? key}) : super(key: key);

  @override
  State<CameraPage> createState() => _CameraPageState();
}

class _CameraPageState extends State<CameraPage> {
  bool flashState = false;
  double ratio = 1;
  double? maxRatio;
  StateSetter? flashZoomState;

  ///  The first rendering is null ，Using the rear camera
  CameraInfo? currentCamera;
  bool isBcakCamera = true;

  ValueNotifier<FlCameraController?> controller =
      ValueNotifier<FlCameraController?>(null);

  ValueNotifier<bool>? hasPreview;

  void listener() {
    if (hasPreview != null &&
        hasPreview!.value != controller.value!.hasPreview) {
      hasPreview!.value = controller.value!.hasPreview;
    }
  }

  @override
  void initState() {
    super.initState();
    hasPreview = ValueNotifier<bool>(false);
  }

  @override
  Widget build(BuildContext context) {
    return ExtendedScaffold(
        body: Stack(children: <Widget>[
      FlCamera(
          onCreateView: (FlCameraController controller) {
            this.controller.value = controller;
            this.controller.value!.addListener(listener);
          },
          uninitialized: Container(
              color: Colors.black,
              alignment: Alignment.center,
              child: const Text('Camera uninitialized',
                  style: TextStyle(color: Colors.white))),
          fit: BoxFit.fitWidth,
          onFlashChanged: (FlashState state) {
            showToast('flash state:$state');
          },
          onZoomChanged: (CameraZoomState zommState) {
            showToast('zoom ratio:${zommState.zoomRatio}');
            if (maxRatio == null && flashZoomState != null) {
              maxRatio = zommState.maxZoomRatio;
              flashZoomState!(() {});
            }
          },
          overlay: Align(
              alignment: Alignment.bottomCenter,
              child: StatefulBuilder(builder: (_, StateSetter state) {
                flashZoomState = state;
                return Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Slider(
                          value: ratio,
                          min: 1,
                          max: maxRatio ?? 20,
                          onChanged: (double value) async {
                            ratio = value;
                            flashZoomState!(() {});
                            controller.value?.setZoomRatio(value);
                          }),
                      IconBox(
                          size: 30,
                          color: flashState
                              ? Colors.white
                              : Colors.white.withOpacity(0.6),
                          padding: const EdgeInsets.fromLTRB(10, 10, 10, 40),
                          icon: flashState ? Icons.flash_on : Icons.flash_off,
                          onTap: () async {
                            final bool? state = await controller.value
                                ?.setFlashMode(flashState
                                    ? FlashState.off
                                    : FlashState.on);
                            if (state == true) {
                              flashState = !flashState;
                              flashZoomState!(() {});
                            }
                          })
                    ]);
              }))),
      Positioned(
          right: 12,
          left: 12,
          top: getStatusBarHeight + 12,
          child: ValueListenableBuilder<FlCameraController?>(
              valueListenable: controller,
              builder: (_, FlCameraController? flCameraController, __) {
                return flCameraController == null
                    ? const SizedBox()
                    : Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                            const BackButton(
                                color: Colors.white, onPressed: pop),
                            Row(children: [
                              ElevatedIcon(
                                  icon: Icons.flip_camera_ios,
                                  onPressed: switchCamera),
                              const SizedBox(width: 12),
                              ElevatedText(
                                  text: 'reset',
                                  onPressed: () =>
                                      controller.value?.resetCamera()),
                              const SizedBox(width: 12),
                              previewButton(flCameraController),
                            ])
                          ]);
              })),
    ]));
  }

  Widget previewButton(FlCameraController flCameraController) {
    return ValueListenableBuilder(
        valueListenable: hasPreview!,
        builder: (_, bool hasPreview, __) {
          return ElevatedText(
              text: !hasPreview ? 'start' : 'stop',
              onPressed: () async {
                if (!hasPreview) {
                  if (flCameraController.previousCamera != null) {
                    await flCameraController
                        .startPreview(flCameraController.previousCamera!);
                  }
                } else {
                  await flCameraController.stopPreview();
                }
              });
        });
  }

  Future<void> switchCamera() async {
    if (controller.value == null) return;
    for (final CameraInfo cameraInfo in controller.value!.cameras!) {
      if (cameraInfo.lensFacing ==
          (isBcakCamera ? CameraLensFacing.front : CameraLensFacing.back)) {
        currentCamera = cameraInfo;
        break;
      }
    }
    await controller.value!.switchCamera(currentCamera!);
    isBcakCamera = !isBcakCamera;
  }

  @override
  void dispose() {
    super.dispose();
    controller.dispose();
    hasPreview?.dispose();
  }
}

class ScannerBoxPage extends StatelessWidget {
  const ScannerBoxPage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const ExtendedScaffold(
        body: FlCamera(
            overlay: ScannerBox(
                borderColor: Colors.blue,
                scannerColor: Colors.blue,
                scannerSize: Size(300, 300),
                scannerStrokeWidth: 2)));
  }
}

class ScannerLinePage extends StatelessWidget {
  const ScannerLinePage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const ExtendedScaffold(
        body: FlCamera(overlay: ScannerLine(color: Colors.blue)));
  }
}
